#!/bin/sh
### ====================================================================== ###
##                                                                          ##
## initPostgres.sh
##                                                                          ##
### ====================================================================== ###


#!/bin/sh
# PostgreSQL cluster init script for Linux

DIRNAME=`dirname $0`
uname -a
. $DIRNAME/setCommonEnv.sh

DB_HOME=$(cd $DB_HOME; pwd -P)

DB_SUPERUSER=postgres
PASSWORD=$1
DATADIR=$DB_HOME/data/
IS_DATADIR_EXISTS=false
WARN=0

warn()
{
		echo $1
		WARN=2
}

_die()
{
		echo $1
		_cleanup
		exit 1
}

_die2()
{
		echo "The service user account '$1' could not be created."
		exit 1
}

_cleanup()
{
		if [ -f "$PWDFILE" ];
		then
		rm -rf "$PWDFILE"
    fi
}

# Create the data directory, and set the appropriate permissions/owership
if [ ! -d "$DATADIR" ];
then
    mkdir -p "$DATADIR" || _die "Failed to create the data directory ($DATADIR)"
else
    IS_DATADIR_EXISTS=true
    echo "SERVER_HOME ::: $(cd $SERVER_HOME; pwd -P)"
    chmod -R 755 $(cd $SERVER_HOME; pwd -P)/tools/postgres/bin/gettimezone*
    MACHINE_TYPE=`uname -m`
    echo "Machine type : $MACHINE_TYPE"
    GETTIMEZONE_BIN=gettimezone
    echo "Getting timezone using :: $SERVER_HOME/tools/postgres/bin/$GETTIMEZONE_BIN"
fi

echo "Data directory already exists :  $IS_DATADIR_EXISTS."

if [ ! -d "$DB_HOME/tmp" ];
then
    mkdir -p "$DB_HOME/tmp" || _die "Failed to create the data directory ($DATADIR)"
fi
    USER_ID=`id -g`
    PWDFILE="$DB_HOME/tmp/.pwd"

if [ ! -z "$PASSWORD" ];
then
    echo "Authentication mode set to MD5"
    touch $PWDFILE || _die "Failed to create the initdb password file (/tmp/initdbpw.$$)"
    chmod 0600 $PWDFILE || _die "Failed to set the permissions on the initdb password file (/tmp/initdbpw.$$)"
    echo "$PASSWORD" > $PWDFILE || _die "Failed to write the initdb password file (/tmp/initdbpw.$$)"
    if [ "$USER_ID" != "0" ];
    then
        AUTHMODE="-A md5 --pwfile $PWDFILE"
    else
        AUTHMODE="-A \"md5\" --pwfile \"$PWDFILE\""
    fi
fi

# Initializing the database cluster
if [ "$USER_ID" != "0" ];
then
    LD_LIBRARY_PATH="/lib:$DB_HOME/extlib:$LD_LIBRARY_PATH"
    if [ "$IS_DATADIR_EXISTS" != "false" ];
    then
        chmod -R 700 $DATADIR
        "$SERVER_HOME/tools/postgres/bin/$GETTIMEZONE_BIN" "$DB_HOME/share/postgresql/" "$DATADIR/postgresql.conf"
    else
        "$DB_HOME/bin/initdb" -U $DB_SUPERUSER -D "$DATADIR" --no-locale -E UTF8 $AUTHMODE || _die "Failed to initialise the database cluster with initdb"
    fi
else
		# Check if the current OS is MAC or Linux
    if [ "$(uname)" = "Darwin" ];
    then
        echo "Current OS is MAC"
				# Create the group if required
        if [ "x`dscl . -list /users|cut -f2 -d' '|grep ^$DB_SUPERUSER\$`" != "x" ];
        then
            HOME_DIR=`su $DB_SUPERUSER -c "echo \\\$HOME"`
            if [ -e $HOME_DIR ]; then
                echo "User account '$DB_SUPERUSER' already exists"
            else
                echo "User account '$DB_SUPERUSER' already exists - fixing non-existent home directory to $2"
                dscl . create /users/$DB_SUPERUSER home $DATADIR
                # Waiting for home directory to be set.
                # Assuming it will get done in 30 sec.
                echo "Waiting for home directory to be set...."
                count=0
                while [ $count -le 15 ]
                do
                    HOME_DIR=`su $DB_SUPERUSER -c "echo \\\$HOME"`
                    if [ -e $HOME_DIR ]; then
                        exit 0
                    fi
                    sleep 2
                    count=`expr $count + 1`
                done
                exit 0
            fi
        else
						#Creating new user id for the new user
            NEWUID=`dscl . list /users uid | awk -F: '{FS=" "; print $2f}' | sort -n | tail -1`
            NEWUID=`expr $NEWUID + 1`
						echo "Creating User account postgres."
            dscl . create /users/$DB_SUPERUSER || _die $DB_SUPERUSER
            dscl . create /users/$DB_SUPERUSER name $DB_SUPERUSER
            dscl . create /users/$DB_SUPERUSER passwd "*"
            dscl . create /users/$DB_SUPERUSER uid $NEWUID
            dscl . create /users/$DB_SUPERUSER gid 1
            dscl . create /users/$DB_SUPERUSER home $DATADIR
            dscl . create /users/$DB_SUPERUSER shell /bin/bash
            dscl . create /users/$DB_SUPERUSER realname "postgres"
						#Hiding added user from logon screen
						defaults write /Library/Preferences/com.apple.loginwindow HiddenUsersList -array-add $DB_SUPERUSER
        fi
        chown -R $DB_SUPERUSER:daemon "$DATADIR" || _die "Failed to set the ownership of the data directory ($DATADIR)"
        chmod -R 700 "$DATADIR"
        chown -R $DB_SUPERUSER:daemon "$DB_HOME/tmp" || _die "Failed to set the ownership of the tmp directory ($DATADIR)"
        chmod -R 700 "$DB_HOME/tmp"
        if [ -f "$PWDFILE" ];
        then
            chown $DB_SUPERUSER:daemon $PWDFILE
        fi
    else
        # Create the group if required
        if ! getent group  $DB_SUPERUSER > /dev/null
        then
            groupadd  $DB_SUPERUSER || _die2 $DB_SUPERUSER
        fi

        # Create the user account if required
        USER_HOME_DIR=$DB_HOME
        if getent passwd  $DB_SUPERUSER > /dev/null
        then
            HOME_DIR=`su $DB_SUPERUSER -c "echo \\\$HOME"`
            if [ -e $HOME_DIR ]; then
                echo "User account '$DB_SUPERUSER' already exists"
            else
                echo "User account '$DB_SUPERUSER' already exists - fixing non-existent home directory to $USER_HOME_DIR"
                usermod -d "$USER_HOME_DIR"  $DB_SUPERUSER
                exit 0
            fi
        else
            useradd -m -c "PostgreSQL" -d "$USER_HOME_DIR" -g  $DB_SUPERUSER  $DB_SUPERUSER || _die $DB_SUPERUSER
            usermod -p "*"  $DB_SUPERUSER
        fi
        chown -R $DB_SUPERUSER:$DB_SUPERUSER "$DATADIR" || _die "Failed to set the ownership of the data directory ($DATADIR)"
        chmod -R 700 "$DATADIR"
        chown -R $DB_SUPERUSER:$DB_SUPERUSER "$DB_HOME/tmp" || _die "Failed to set the ownership of the tmp directory ($DATADIR)"
        chmod -R 700 "$DB_HOME/tmp"
        if [ -f "$PWDFILE" ];
        then
            chown $DB_SUPERUSER:$DB_SUPERUSER $PWDFILE
        fi
    fi
    echo "IS_DATADIR_EXISTS ::: $IS_DATADIR_EXISTS"
    if [ "$IS_DATADIR_EXISTS" != "false" -a -f "$DATADIR/postgresql.conf" ];
    then
        su $DB_SUPERUSER -c "\"$SERVER_HOME/tools/postgres/bin/$GETTIMEZONE_BIN\" \"$DB_HOME/share/postgresql/\" \"$DATADIR/postgresql.conf\""
    else
        su - $DB_SUPERUSER -c "LD_LIBRARY_PATH=\"/lib:$DB_HOME/extlib:$LD_LIBRARY_PATH\" \"$DB_HOME/bin/initdb\" -U \"$DB_SUPERUSER\" -D \"$DATADIR\" --no-locale -E UTF8 $AUTHMODE" || _die "Failed to initialise the database cluster with initdb"
    fi
fi
_cleanup
echo "$0 ran to completion"
exit $WARN
