/**
 *This file taken from : https://gist.github.com/msqr/3202712
 *
 * Licensed under MIT
 */
var gauge = function(container, configuration) {
	var that = {};
	var config = {
		size						: 200,
		clipWidth					: 200,
		clipHeight					: 110,
		ringInset					: 20,
		ringWidth					: 20,
		
		pointerWidth					: 8,
		pointerTailLength				: 4,
		pointerHeadLengthPercent			: 0.8,
		
		minValue					: 0,
		maxValue					: 10,
		
		minAngle					: -90,
		maxAngle					: 90,
		
		transitionMs					: 750,
		
		majorTicks					: 5,
		labelFormat					: d3.format(',g'),//No I18N
		labelInset					: 10,
	
		unit						: '%',	//No I18N
		arcColorFn					: d3.interpolateHsl(d3.rgb('#e8e2ca'), d3.rgb('#3e6c0a'))//No I18N
	};

	//e8e2ca & 3e6c0a, A7B0C7 & 072D94
	var range = undefined;
	var r = undefined;
	var pointerHeadLength = undefined;
	var value = 0;
	
	var svg = undefined;
	var textElement = undefined;
	var arc = undefined;
	var scale = undefined;
	var ticks = undefined;
	var tickData = undefined;
	var pointer = undefined;

	var donut = d3.layout.pie();
	
	function deg2rad(deg) {
		return deg * Math.PI / 180;
	}
	
	function newAngle(d) {
		var ratio = scale(d);
		var newAngle = config.minAngle + (ratio * range);
		return newAngle;
	}
	
	function configure(configuration) {
		var prop = undefined;
		for ( prop in configuration ) {
			config[prop] = configuration[prop];
		}
		
		range = config.maxAngle - config.minAngle;
		r = config.size / 2;
		pointerHeadLength = Math.round(r * config.pointerHeadLengthPercent);

		// a linear scale that maps domain values to a percent from 0..1
		scale = d3.scale.linear().range([0,1]).domain([config.minValue, config.maxValue]);
			
		ticks = scale.ticks(config.majorTicks);
		tickData = d3.range(config.majorTicks).map(function() {return 1/config.majorTicks;});
		
		arc = d3.svg.arc()
			.innerRadius(r - config.ringWidth - config.ringInset)
			.outerRadius(r - config.ringInset)
			.startAngle(function(d, i) {
				var ratio = d * i;
				return deg2rad(config.minAngle + (ratio * range));
			})
			.endAngle(function(d, i) {
				var ratio = d * (i+1);
				return deg2rad(config.minAngle + (ratio * range));
			});
	}
	that.configure = configure;
	
	function centerTranslation() {
		return 'translate('+r +','+ r +')';//No I18N
	}
	
	function isRendered() {
		return (svg !== undefined);
	}
	that.isRendered = isRendered;
	
	function render(newValue) {
		svg = d3.select(container)
			.append('svg:svg')//No I18N
				.attr('class', 'gauge')//No I18N
				.attr('width', config.clipWidth)//No I18N
				.attr('height', config.clipHeight);//No I18N
		
		var centerTx = centerTranslation();
		
		arcs = svg.append('g').attr('class', 'arc').attr('transform', centerTx);//No I18N
		
		arcs.selectAll('path').data(tickData).enter().append('path').attr('fill', function(d, i) {return config.arcColorFn(d * i);}).attr('d', arc);//No I18N
	
		textElement = arcs.append('text').attr("font-size", "18px").attr('x',-10).attr('y',30);//No I18N

		var lg = svg.append('g')//No I18N
				.attr('class', 'label')//No I18N
				.attr('transform', centerTx);//No I18N
		lg.selectAll('text')//No I18N
				.data(ticks)
			.enter().append('text')//No I18N
				.attr('transform', function(d) {//No I18N
					var ratio = scale(d);
					var newAngle = config.minAngle + (ratio * range);//No I18N
					return 'rotate(' +newAngle +') translate(0,' +(config.labelInset - r) +')';//No I18N
				})
				.text(config.labelFormat);

		var lineData = [ [config.pointerWidth / 2, 0], 
						[0, -pointerHeadLength],
						[-(config.pointerWidth / 2), 0],
						[0, config.pointerTailLength],
						[config.pointerWidth / 2, 0] ];
		var pointerLine = d3.svg.line().interpolate('monotone');//No I18N
		var pg = svg.append('g').data([lineData])//No I18N
				.attr('class', 'pointer')//No I18N
				.attr('transform', centerTx);//No I18N
				
		pointer = pg.append('path')//No I18N
			.attr('d', pointerLine/*function(d) { return pointerLine(d) +'Z';}*/ )//No I18N
			.attr('transform', 'rotate(' +config.minAngle +')');		//No I18N
			
		update(newValue === undefined ? 0 : newValue);
	}
	that.render = render;
	
	function update(newValue, newConfiguration) {
		if ( newConfiguration  !== undefined) {
			configure(newConfiguration);
		}
	
		textElement.text(function(d){return (d3.round(newValue)+" "+config.unit);});
		if(config.dialName === "AvailabilityGaugeDiv" || config.dialName === "interfaceAvailabilityGaugeDiv")
		{
			/*if(Math.round(newValue) === 100 || Math.round(newValue) === "100")
			{

				*/textElement.attr("fill","green");//No I18N
			/*}
			else
			{
				textElement.attr("fill","red");//No I18N
			}*/
		}
		else
		{
			/*if(Math.round(newValue) > 80)
			{
				textElement.attr("fill","red");//No I18N
			}
			else if(Math.round(newValue) <=80 && Math.round(newValue) > 60)
			{
				textElement.attr("fill","orange");//No I18N
			}
			else
			{*/
				textElement.attr("fill","green");//No I18N
			//}
		}


		var ratio = scale(newValue);
		var newAngle = config.minAngle + (ratio * range);
		pointer.transition()
			.duration(config.transitionMs)
			//.ease('elastic')
			.attr('transform', 'rotate(' +newAngle +')');//No I18N
	}
	that.update = update;

	configure(configuration);
	
	return that;
};
